﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace ThreadUpdateWithSelection
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        LiveDataViewModel vm;
        public MainWindow()
        {
            InitializeComponent();

            vm = new LiveDataViewModel();
            DataContext = vm;

            Closing += MainWindow_Closing;
        }

        void MainWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            vm.StopUpdates();
        }
    }

    
    public class ChartData : ViewModelBase
    {
        private DateTime date;

        public DateTime Date 
        {
            get { return date; }
            set { date = value; OnPropertyChanged("Date"); }
        }

        private double val1;
        public double Value1
        {
            get { return val1; }
            set { val1 = value; OnPropertyChanged("Value1"); }
        }


        private double val2;
        public double Value2
        {
            get { return val2; }
            set { val2 = value; OnPropertyChanged("Value2"); }
        }

        private double val3;
        public double Value3
        {
            get { return val3; }
            set { val3 = value; OnPropertyChanged("Value3"); }
        }   
        
       

        public static ObservableCollection<ChartData> GenerateData(int count)
        {
            ObservableCollection<ChartData> data = new ObservableCollection<ChartData>();

            Random r = new Random();

            ChartData data0 = new ChartData
            {
                Date = DateTime.Now, 
                Value1 = r.Next(20, 40),
                Value2 = r.Next(20, 40),
                Value3 = r.Next(20, 40),
            };

            data.Add(data0);

            for (int i = 1; i < count; i++)
            {
                ChartData dat = new ChartData
                {
                    Date = DateTime.Now.AddDays(i), 
                };

                if (r.NextDouble() > .5)
                {
                    dat.Value1 = data[i - 1].Value1 + r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 + r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 + r.NextDouble();
                }
                else
                {
                    dat.Value1 = data[i - 1].Value1 - r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 - r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 - r.NextDouble();
                }

                data.Add(dat);
            }

            return data;
        }
    }


    public class LiveDataViewModel
    {
        private ObservableCollection<ChartData> data;
        Thread updateThread;

        public LiveDataViewModel()
        {
            Data = ChartData.GenerateData(1000);

            updateThread = new Thread(new ThreadStart(() =>
                {
                    while (true)
                    {
                        Thread.Sleep(1000);

                        Random r = new Random();

                        foreach (var dat in Data)
                        {
                            dat.Date = Data.Last().Date.AddDays(r.Next(100));
                            dat.Value1 = r.Next(100) + r.NextDouble();
                            dat.Value2 = r.Next(100) + r.NextDouble();
                            dat.Value3 = r.Next(100) + r.NextDouble();
                        }
                    }                   
                }));

            updateThread.Start();
        }

        public ObservableCollection<ChartData> Data
        {
            get { return data; }
            set { data = value; }
        }

        internal void StopUpdates()
        {
            updateThread.Abort();
        }
    }


    
    public class ViewModelBase : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }		
				
}
